@@ -27,6 +27,8 @@
 #' \code{resetMV}: Boolean specifying whether to begin recording posterior sample chains anew. This argument is only considered when using \code{reset = FALSE}.  Specifying \code{reset = FALSE, resetMV = TRUE} allows the MCMC algorithm to continue running from where it left off, but without appending the new posterior samples to the already existing samples, i.e. all previously obtained samples will be erased. This option can help reduce memory usage during burn-in (default = FALSE).
 #'
 #' \code{resetWAIC}: Boolean specifying whether to reset the WAIC summary statistics to their initial states and thereby begin the WAIC calculation anew (default = TRUE). Specifying \code{resetWAIC = FALSE} allows the WAIC calculation to continue running from where it left off. 
+#'
+#' \code{initializeModel}: Boolean specifying whether to run the initializeModel routine on the underlying model object, prior to beginning MCMC sampling (default = TRUE).
 #' 
 #' \code{chain}: Integer specifying the MCMC chain number.  The chain number is passed to each MCMC sampler's before_chain and after_chain methods.  The value for this argument is specified automatically from invocation via runMCMC, and genernally need not be supplied when calling mcmc$run (default = 1).
 
@@ -146,7 +148,7 @@ buildMCMC <- nimbleFunction(
             }
         }
 
-        ## build sampler functions.
+        ## build sampler functions
         samplerFunctions <- nimbleFunctionList(sampler_BASE)
 
         predictiveNodeIDs <- conf$model$getPredictiveNodeIDs()
@@ -224,6 +226,7 @@ buildMCMC <- nimbleFunction(
         thin                  = double(default = -1),
         thin2                 = double(default = -1),
         resetWAIC             = logical(default = TRUE),
+        initializeModel       = logical(default = TRUE),
         chain                 = integer(default =  1)) {
         if(niter < 0)       stop('cannot specify niter < 0')
         if(nburnin < 0)     stop('cannot specify nburnin < 0')
@@ -235,7 +238,7 @@ buildMCMC <- nimbleFunction(
             if(thinToUseVec[iThin] < 1)   stop('cannot use thin < 1')
             if(thinToUseVec[iThin] != floor(thinToUseVec[iThin]))   stop('cannot use non-integer thin')
         }
-        my_initializeModel$run()
+        if(initializeModel)   my_initializeModel$run()
         nimCopy(from = model, to = mvSaved, row = 1, logProb = TRUE)
         if(reset) {
             samplerTimes <<- numeric(length(samplerFunctions) + 1)       ## default inititialization to zero
