@@ -7,6 +7,7 @@ otherDSLcalls <- c("{",
                    "returnType",
                    "seq_along",
                    "double",
+                   "character",
                    "rankSample",
                    "new",
                    "nimEigen",
@@ -44,9 +45,7 @@ nimKeyWords <- list(copy = 'nimCopy',
                     max.bound = 'carMaxBound',
                     derivs = 'nimDerivs')
 
-distsNotAllowedInAD <- c(
-  paste0('d', c('interval', 'constraint'))
-)
+# distsNotAllowedInAD is defined near the end of distributions_inputList.R
 
 fxnsNotAllowedInAD <- c(
   '%%',
