@@ -66,20 +66,24 @@ class NimOptimProblem {
           lower_(lower),
           upper_(upper),
           control_(control),
-          hessian_(hessian) {}
+          hessian_(hessian),
+          gr_provided_(false),
+          he_provided_(false){}
     nimSmartPtr<OptimResultNimbleList> solve(NimArr<1, double>& par);
+    static double fn(int, double*, void*);
+    static void gr(int, double*, double*, void*);
+    static void he(int, double*, double*, void*);
 
    private:
     // These function and gradient callbacks for R's optim() where `this` is
     // passed in as the final argument `void * ex`.
-    static double fn(int, double*, void*);
-    static void gr(int, double*, double*, void*);
     void calc_hessian(NimArr<1, double> par,
 		      NimArr<2, double> &hessian);
    protected:
     // These are callbacks used internally by fn() and gr().
     virtual double function() = 0;
     virtual void gradient() = 0;
+    virtual void hessian_callback() = 0;
 
     // Problem parameters.
     const std::string& method_;
@@ -88,9 +92,13 @@ class NimOptimProblem {
     nimSmartPtr<OptimControlNimbleList> control_;
     bool hessian_;
 
+    bool gr_provided_;
+    bool he_provided_;
+
     // Temporaries.
-    NimArr<1, double> par_;  // Argument for fn() and gr().
+    NimArr<1, double> par_;  // Argument for fn() and gr() and he().
     NimArr<1, double> ans_;  // Result of gradient.
+    NimArr<2, double> ans_hessian_;  // Result of hessian.
     // entries in the control_ list that might need to change
     // dynamically with each call. These cannot change control_
     // in place because it might be shared with other calls to nimOptim.
@@ -99,6 +107,12 @@ class NimOptimProblem {
     NimArr<1, double> working_ndeps;
 };
 
+extern "C" {
+    SEXP CALL_NimOptimProblem_fn(SEXP Sx, SEXP Sext_ptr);
+    SEXP CALL_NimOptimProblem_gr(SEXP Sx, SEXP Sext_ptr);
+    SEXP CALL_NimOptimProblem_he(SEXP Sx, SEXP Sext_ptr);
+}
+
 template <class Fn>
 class NimOptimProblem_Fun : public NimOptimProblem {
    public:
@@ -111,11 +125,17 @@ class NimOptimProblem_Fun : public NimOptimProblem {
    protected:
     virtual double function() { return fn_(par_); }
     virtual void gradient();  // Uses finite difference approximation.
+    virtual void hessian_callback();
 
    private:
     Fn fn_;
 };
 
+template <class Fn>
+void NimOptimProblem_Fun<Fn>::hessian_callback() {
+  Rprintf("Illegally trying to calculate hessian when not provided.\n");
+}
+
 template <class Fn>
 void NimOptimProblem_Fun<Fn>::gradient() {
   // When gradient() is called (from NimOptimProblem::gr),
@@ -142,23 +162,29 @@ void NimOptimProblem_Fun<Fn>::gradient() {
             // on the fn scale and so are upper_ and lower_.
             const double h_pos = std::min(h, upper_[i] - par_[i]);
             const double h_neg = std::min(h, par_[i] - lower_[i]);
+            // We have to defensively copy par_h = par_ each time in case fn_ modifies the length or values of par_h
+            par_h = par_;
             par_h[i] = par_[i] + h_pos;
             const double pos = fn_(par_h);
+            par_h = par_;
             par_h[i] = par_[i] - h_neg;
             const double neg = fn_(par_h);
-            par_h[i] = par_[i];
+            //par_h[i] = par_[i];
             ans_[i] = parscale[i] * (pos - neg) / (h_pos + h_neg);
         }
     } else {
         // Unconstrained optimization.
         for (int i = 0; i < n; ++i) {
             const double h = working_ndeps[i]*parscale[i];
+            par_h = par_;
             par_h[i] = par_[i] + h;
             const double pos = fn_(par_h);
+            par_h = par_;
             par_h[i] = par_[i] - h;
             const double neg = fn_(par_h);
-            par_h[i] = par_[i];
-            ans_[i] = (pos - neg) / (2 * working_ndeps[i]);
+            //par_h[i] = par_[i];
+            ans_[i] = (pos - neg) / (2 * working_ndeps[i]); // this is implicitly multiplied by parscale[i]
+            // because instead of dividing by h we are dividing by working_ndeps[i] = h/parscale[i]
         }
     }
     // dividing the answer by fnscale is done by the calling
@@ -174,7 +200,8 @@ class NimOptimProblem_Fun_Grad : public NimOptimProblem {
                              bool hessian)
         : NimOptimProblem(method, lower, upper, control, hessian),
           fn_(fn),
-          gr_(gr) {}
+          gr_(gr)
+          {NimOptimProblem::gr_provided_ = true;}
 
    protected:
     virtual double function() { return fn_(par_); }
@@ -182,27 +209,60 @@ class NimOptimProblem_Fun_Grad : public NimOptimProblem {
       // This should return gradient on par/parscale.
       // But gr_ calculates the gradient wrt par.
       // So we have to multiply by parscale.
+      const int n = par_.dimSize(0);
       ans_ = gr_(par_);
       double* parscale = working_parscale.getPtr();
-      const int n = par_.dimSize(0);
       for (int i = 0; i < n; ++i) {
         ans_[i] *= parscale[i];
       }
     }
-
+  virtual void hessian_callback();
    private:
     Fn fn_;
     Gr gr_;
 };
 
+template <class Fn, class Gr>
+void NimOptimProblem_Fun_Grad<Fn, Gr>::hessian_callback() {
+  Rprintf("Illegally trying to calculate hessian when not provided.\n");
+}
+
+
+template <class Fn, class Gr, class He>
+class NimOptimProblem_Fun_Grad_Hess : public NimOptimProblem_Fun_Grad<Fn, Gr> {
+   public:
+    NimOptimProblem_Fun_Grad_Hess(Fn fn, Gr gr, He he, const std::string& method,
+                             NimArr<1, double>& lower, NimArr<1, double>& upper,
+                             nimSmartPtr<OptimControlNimbleList> control,
+                             bool hessian)
+        : NimOptimProblem_Fun_Grad<Fn, Gr>(fn, gr, method, lower, upper, control, hessian),
+          he_(he)
+  {NimOptimProblem::he_provided_ = true;}
+
+  protected:
+  virtual void hessian_callback() {
+    const int n = NimOptimProblem::par_.dimSize(0);
+    NimOptimProblem::ans_hessian_ = he_();
+    double* parscale = NimOptimProblem::working_parscale.getPtr();
+    for(int i = 0; i < n; ++i) {
+      for(int j = 0; j < n; ++j) {
+        NimOptimProblem::ans_hessian_[i,j] *= parscale[i] * parscale[j];
+      }
+    }
+  }
+  private:
+    He he_;
+};
 // ---------------------------------------------------------------------------
 // These nimOptim() and nimOptimDefaultControl() will appear in generated code
 
 nimSmartPtr<OptimControlNimbleList> nimOptimDefaultControl();
 
+// no he
+// lower and upper are vectors
 template <class Fn, class Gr>
 inline nimSmartPtr<OptimResultNimbleList> nimOptim(
-    NimArr<1, double>& par, Fn fn, Gr gr, const std::string& method,
+    NimArr<1, double>& par, Fn fn, Gr gr, const char* he, const std::string& method,
     NimArr<1, double>& lower, NimArr<1, double>& upper,
     nimSmartPtr<OptimControlNimbleList> control, bool hessian) {
     return NimOptimProblem_Fun_Grad<Fn, Gr>(fn, gr, method, lower, upper,
@@ -210,10 +270,11 @@ inline nimSmartPtr<OptimResultNimbleList> nimOptim(
         .solve(par);
 }
 
-// This handles the special case where gr is not specified (i.e. is "NULL").
+// no gr, no he
+// lower and upper are vectors
 template <class Fn>
 inline nimSmartPtr<OptimResultNimbleList> nimOptim(
-    NimArr<1, double>& par, Fn fn, const char* gr, const std::string& method,
+    NimArr<1, double>& par, Fn fn, const char* gr, const char* he, const std::string& method,
     NimArr<1, double>& lower, NimArr<1, double>& upper,
     nimSmartPtr<OptimControlNimbleList> control, bool hessian) {
     NIM_ASSERT1(
@@ -223,10 +284,11 @@ inline nimSmartPtr<OptimResultNimbleList> nimOptim(
         .solve(par);
 }
 
-// TODO Handle double -> vector conversion in R instead of here.
+// no he
+// lower and upper are scalars
 template <class Fn, class Gr>
 inline nimSmartPtr<OptimResultNimbleList> nimOptim(
-    NimArr<1, double>& par, Fn fn, Gr gr, const std::string& method,
+    NimArr<1, double>& par, Fn fn, Gr gr, const char* he, const std::string& method,
     double lower, double upper, nimSmartPtr<OptimControlNimbleList> control,
     bool hessian) {
     NimArr<1, double> lower_vector;
@@ -238,11 +300,11 @@ inline nimSmartPtr<OptimResultNimbleList> nimOptim(
         .solve(par);
 }
 
-// TODO Handle double -> vector conversion in R instead of here.
-// This handles the special case where gr is not specified (i.e. is "NULL").
+// no gr, no he
+// lower and upper are scalars
 template <class Fn>
 inline nimSmartPtr<OptimResultNimbleList> nimOptim(
-    NimArr<1, double>& par, Fn fn, const char* gr, const std::string& method,
+    NimArr<1, double>& par, Fn fn, const char* gr, const char* he, const std::string& method,
     double lower, double upper, nimSmartPtr<OptimControlNimbleList> control,
     bool hessian) {
     NIM_ASSERT1(
@@ -257,4 +319,33 @@ inline nimSmartPtr<OptimResultNimbleList> nimOptim(
         .solve(par);
 }
 
+// all provided
+// lower and upper are scalars
+template <class Fn, class Gr, class He>
+inline nimSmartPtr<OptimResultNimbleList> nimOptim(
+    NimArr<1, double>& par, Fn fn, Gr gr, He he, const std::string& method,
+    double lower, double upper, nimSmartPtr<OptimControlNimbleList> control,
+    bool hessian) {
+    NimArr<1, double> lower_vector;
+    lower_vector.initialize(lower, true, 1);
+    NimArr<1, double> upper_vector;
+    upper_vector.initialize(upper, true, 1);
+    return NimOptimProblem_Fun_Grad_Hess<Fn, Gr, He>(fn, gr, he, method, lower_vector,
+                                                     upper_vector, control, hessian)
+        .solve(par);
+}
+
+// all provided
+// lower and upper are vectors
+template <class Fn, class Gr, class He>
+inline nimSmartPtr<OptimResultNimbleList> nimOptim(
+    NimArr<1, double>& par, Fn fn, Gr gr, He he, const std::string& method,
+    NimArr<1, double>& lower, NimArr<1, double>& upper,
+    nimSmartPtr<OptimControlNimbleList> control,
+    bool hessian) {
+    return NimOptimProblem_Fun_Grad_Hess<Fn, Gr, He>(fn, gr, he, method, lower,
+                                                     upper, control, hessian)
+        .solve(par);
+}
+
 #endif  // __NIMBLE_NIMOPTIM_H
