@@ -45,7 +45,7 @@ test_that("Warning message works for call not supported for derivs.", {
   expect_identical(output, character())
 })
 
-test_that("Warning messages work for checking if a user-defined distribution supports derivs when it must.", {
+test_that("Warning messages work for checking if a user-defined distribution supports derivs when it must (1).", {
   # Case where both user-defined dist and model have buildDerivs TRUE, so no warning is emitted.
   dmyexp <- nimbleFunction(
     run = function(x = double(0), rate = double(0, default = 1), log = integer(0, default = 0)) {
@@ -57,7 +57,7 @@ test_that("Warning messages work for checking if a user-defined distribution sup
         return(exp(logProb))
       }
     }, buildDerivs = TRUE)
-  temporarilyAssignInGlobalEnv(dmyexp)
+#  temporarilyAssignInGlobalEnv(dmyexp)
   
   rmyexp <- nimbleFunction(
     run = function(n = integer(0), rate = double(0, default = 1)) {
@@ -67,7 +67,7 @@ test_that("Warning messages work for checking if a user-defined distribution sup
       return(-log(1-dev) / rate)
     }
   )
-  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+#  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
   
   code1 <- nimbleCode({
     for(i in 1:3) {
@@ -89,7 +89,10 @@ test_that("Warning messages work for checking if a user-defined distribution sup
     m <- nimbleModel(code1)
   )
   expect_false(any(grepl("deriv", msgs)))
+  deregisterDistributions("dmyexp")
+})
 
+test_that("Warning messages work for checking if a user-defined distribution supports derivs when it must (2).", {
   # Case where user-defined dist has buildDerivs FALSE (by omission) but model needs derivs, so a warning is emitted.
   ## deregisterDistributions("dmyexp")
   dmyexp <- nimbleFunction(
@@ -102,7 +105,14 @@ test_that("Warning messages work for checking if a user-defined distribution sup
         return(exp(logProb))
       }
     })
-  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+#  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+  code1 <- nimbleCode({
+    for(i in 1:3) {
+      y1[i] ~ dmyexp(rate = r1)
+    }
+    r1 <- 1 / s1
+    s1 ~ dunif(0, 100)
+  })
 
   msgs <- capture_messages(
     m <- nimbleModel(code1, buildDerivs = TRUE)
@@ -110,15 +120,16 @@ test_that("Warning messages work for checking if a user-defined distribution sup
   expect_true(any(grepl("Distribution dmyexp", msgs) &
                     grepl("derivatives", msgs)))
 
-
-
 # Follow-on case where model does not need derivatives so no message is needed.
   msgs <- capture_messages(
     m <- nimbleModel(code1, buildDerivs = FALSE)
   )
   expect_false(any(grepl("Distribution dmyexp", msgs) &
-                     grepl("derivatives", msgs)))
+                   grepl("derivatives", msgs)))
+  deregisterDistributions("dmyexp")
+})
 
+test_that("Warning messages work for checking if a user-defined distribution supports derivs when it must (3).", {
   # Case where user-defined dist has buildDerivs FALSE (by literal) but model needs derivs, so a warning is emitted.
   ## deregisterDistributions("dmyexp")
   dmyexp <- nimbleFunction(
@@ -131,7 +142,15 @@ test_that("Warning messages work for checking if a user-defined distribution sup
         return(exp(logProb))
       }
     }, buildDerivs = FALSE)
-  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+#  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+  code1 <- nimbleCode({
+    for(i in 1:3) {
+      y1[i] ~ dmyexp(rate = r1)
+    }
+    r1 <- 1 / s1
+    s1 ~ dunif(0, 100)
+  })
+
   msgs <- capture_messages(
     m <- nimbleModel(code1, buildDerivs = TRUE)
   )
@@ -143,7 +162,10 @@ test_that("Warning messages work for checking if a user-defined distribution sup
   )
   expect_false(any(grepl("Distribution dmyexp", msgs) &
                      grepl("derivatives", msgs)))
+  deregisterDistributions("dmyexp")
+})
 
+test_that("Warning messages work for checking if a user-defined distribution supports derivs when it must (4).", {
   # Case where user-defined dist has buildDerivs FALSE (by empty list) but model needs derivs, so a warning is emitted.
   ## deregisterDistributions("dmyexp")
   dmyexp <- nimbleFunction(
@@ -156,23 +178,37 @@ test_that("Warning messages work for checking if a user-defined distribution sup
         return(exp(logProb))
       }
     }, buildDerivs = list())
-  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+#  temporarilyAssignInGlobalEnv(rmyexp, replace = TRUE)
+  code1 <- nimbleCode({
+    for(i in 1:3) {
+      y1[i] ~ dmyexp(rate = r1)
+    }
+    r1 <- 1 / s1
+    s1 ~ dunif(0, 100)
+  })
+
   msgs <- capture_messages(
     m <- nimbleModel(code1, buildDerivs = TRUE)
   )
   expect_true(any(grepl("Distribution dmyexp", msgs) &
                     grepl("derivatives", msgs)))
   # Follow-on case where model does not need derivatives so no message is needed.
+  code1 <- nimbleCode({
+    for(i in 1:3) {
+      y1[i] ~ dmyexp(rate = r1)
+    }
+    r1 <- 1 / s1
+    s1 ~ dunif(0, 100)
+  })
   msgs <- capture_messages(
     m <- nimbleModel(code1, buildDerivs = FALSE)
   )
   expect_false(any(grepl("Distribution dmyexp", msgs) &
                      grepl("derivatives", msgs)))
-
+  deregisterDistributions("dmyexp")
 })
 
 test_that("Warning message works for buildDerivs not set for methods being differentiated.", {
-
     expect_silent(
         derivs_nf <- nimbleFunction(
             setup = function(model, with_respect_to_nodes, calc_nodes) {},
