@@ -1,3 +1,4 @@
+
 # Tests of Laplace approximation
 source(system.file(file.path('tests', 'testthat', 'test_utils.R'), package = 'nimble'))
 source(system.file(file.path('tests', 'testthat', 'AD_test_utils.R'), package = 'nimble'))
@@ -29,39 +30,46 @@ check_laplace_alternative_methods <- function(cL, # compiled laplace algorithm
     reset()
     expect_wrapper(opt <- cL$findMLE())
   }
+
+  cL$updateSettings(useInnerCache=FALSE)
+  #cL$setInnerCache(FALSE) ## Recalculate inner optim to check starting values. Will ensure errors are printed on summary.
+
   if(missing(summ_orig)){
     expect_wrapper(summ_orig <- cL$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
   }
   if(missing(summ_trans)){
     expect_wrapper(summ_trans <- cL$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
   }
-  ref_method <- cL$getMethod()
+  ref_method <- cL$computeMethod_ #cL$getMethod()
   for(method in methods) {
     if(method != ref_method) {
       reset()
-      if(expected_no_re)
-          expect_output(cL$setMethod(method), "no random effects") else cL$setMethod(method)
+      cL$updateSettings(computeMethod=method)
+      ##      if(expected_no_re)
+      ##          expect_output(cL$setMethod(method), "no random effects") else cL$setMethod(method)
       expect_wrapper(opt_alt <- cL$findMLE())
       expect_equal(opt$par, opt_alt$par, tolerance = 0.01)
-      expect_equal(opt$value, opt_alt$value, tolerance = 1e-7)
+      expect_equal(opt$value, opt_alt$value, tolerance = 1e-4)
       tryResult <- try({
-      expect_wrapper(summ_orig_alt <- cL$summary(opt_alt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
-      expect_wrapper(summ_trans_alt <- cL$summary(opt_alt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
+          expect_wrapper(summ_orig_alt <- cL$summary(opt_alt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
+          expect_wrapper(summ_trans_alt <- cL$summary(opt_alt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = TRUE))
       })
       if(inherits(tryResult, 'try-error')) {
+          print("cL$summary failing.")
           print(class(cL))
           print(cL)
+      } else {
+          expect_equal(summ_orig$params$estimates, summ_orig_alt$params$estimates, tol = 1e-4)
+          expect_equal(summ_orig$randomEffects$estimates, summ_orig_alt$randomEffects$estimates, tol = 1e-4)
+          expect_equal(summ_orig$params$stdErrors, summ_orig_alt$params$stdErrors, tol = 1e-4)
+          expect_equal(summ_orig$randomEffects$stdErrors, summ_orig_alt$randomEffects$stdErrors, tol = 1e-4)
+          expect_equal(summ_orig$vcov, summ_orig_alt$vcov, tol = 1e-4)
+          expect_equal(summ_trans$params$estimates, summ_trans_alt$params$estimates, tol = 1e-4)
+          expect_equal(summ_trans$randomEffects$estimates, summ_trans_alt$randomEffects$estimates, tol = 1e-4)
+          expect_equal(summ_trans$params$stdErrors, summ_trans_alt$params$stdErrors, tol = 1e-4)
+          expect_equal(summ_trans$randomEffects$stdErrors, summ_trans_alt$randomEffects$stdErrors, tol = 1e-4)
+          expect_equal(summ_trans$vcov, summ_trans_alt$vcov, tol = 1e-4)
       }
-      expect_equal(summ_orig$params$estimates, summ_orig_alt$params$estimates, tol = 1e-5)
-      expect_equal(summ_orig$randomEffects$estimates, summ_orig_alt$randomEffects$estimates, tol = 1e-5)
-      expect_equal(summ_orig$params$stdErrors, summ_orig_alt$params$stdErrors, tol = 1e-5)
-      expect_equal(summ_orig$randomEffects$stdErrors, summ_orig_alt$randomEffects$stdErrors, tol = 1e-5)
-      expect_equal(summ_orig$vcov, summ_orig_alt$vcov, tol = 1e-5)
-      expect_equal(summ_trans$params$estimates, summ_trans_alt$params$estimates, tol = 1e-5)
-      expect_equal(summ_trans$randomEffects$estimates, summ_trans_alt$randomEffects$estimates, tol = 1e-5)
-      expect_equal(summ_trans$params$stdErrors, summ_trans_alt$params$stdErrors, tol = 1e-5)
-      expect_equal(summ_trans$randomEffects$stdErrors, summ_trans_alt$randomEffects$stdErrors, tol = 1e-5)
-      expect_equal(summ_trans$vcov, summ_trans_alt$vcov, tol = 1e-5)
     }
   }
   invisible(NULL)
@@ -85,7 +93,7 @@ test_that("Laplace simplest 1D works", {
   cmLaplaceNoSplit <- cL$mLaplaceNoSplit
 
   opt <- cmLaplace$findMLE()
-  expect_equal(opt$par, 4, tol = 1e-4) # optim's reltol is about 1e-8 but that is for the value, not param.
+  expect_equal(opt$par, 4, tol = 1e-6) # tolerance was reduced in this test when we switched to nlminb
   # V[a] = 9
   # V[y] = 9 + 4 = 13
   # Cov[a, y] = V[a] = 9 (not needed)
@@ -236,14 +244,14 @@ test_that("Laplace simplest 1D (constrained) with multiple data works", {
   # tmbres <- nlminb(obj$par, obj$fn, obj$gr)
   # tmbrep <- sdreport(obj, getJointPrecision = TRUE)
   # tmbvcov <- inverse(tmbrep$jointPrecision)
-  expect_equal(opt$par, 0.2895238, tol = 1e-4)
+  expect_equal(opt$par, 0.2895238, tol = 1e-3)
   expect_equal(opt$value, -10.47905, tol = 1e-7)
-  expect_equal(summ$randomEffects$estimates, -0.005608619, tol = 1e-4)
+  expect_equal(summ$randomEffects$estimates, -0.005608619, tol = 1e-3)
   vcov <- matrix(c(2.741033, -1.628299, -1.628299, 1.414499), nrow = 2, byrow = TRUE)
   expect_equal(summ$vcov, vcov, 2e-3)
   # Check covariance matrix for params only
   summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ2$vcov, vcov[1,1,drop=FALSE], tol=1e-5)
+  expect_equal(summ2$vcov, vcov[1,1,drop=FALSE], tol=1e-3)
   
   
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
@@ -307,24 +315,28 @@ test_that("Laplace simplest 1D (constrained) with deterministic intermediates an
   # tmbres <- nlminb(obj$par, obj$fn, obj$gr)
   # tmbrep <- sdreport(obj, getJointPrecision = TRUE)
   # tmbvcov <- inverse(tmbrep$jointPrecision)
-  expect_equal(opt$par, -2.639534, 1e-7)
-  expect_equal(opt$value, -10.47905, tol = 1e-7)
-  expect_equal(summ$randomEffects$estimates, 1.603742, tol = 1e-7)
+  expect_equal(opt$par, -2.639534, 1e-4)
+  expect_equal(opt$value, -10.47905, tol = 1e-5)
+  expect_equal(summ$randomEffects$estimates, 1.603742, tol = 1e-4)
   vcov <- matrix(c(10.967784, -3.258191, -3.258191, 1.415167), nrow = 2, byrow = TRUE)
-  expect_equal(summ$vcov, vcov, 1e-7)
+  expect_equal(summ$vcov, vcov, 2e-3)
   # Check covariance matrix for params only
   summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ2$vcov, vcov[1,1,drop=FALSE], tol=1e-7)
+  expect_equal(summ2$vcov, vcov[1,1,drop=FALSE], tol=2e-3)
   
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
   optNoSplit <- cmLaplaceNoSplit$findMLE()
   expect_equal(opt$par, optNoSplit$par, tol = 1e-2)
-  expect_equal(opt$value, optNoSplit$value, tol = 1e-7)
+  expect_equal(opt$value, optNoSplit$value, tol = 1e-4)
   check_laplace_alternative_methods(cmLaplace, cm, m, opt)
   check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, optNoSplit)
 })
 
 test_that("Laplace 1D with deterministic intermediates works", {
+  # Note this test has some slop. In old versions there were inner optimization
+  # warnings issued for this case. So we turned on warnings and checked for them.
+  # Now the warnings aren't issued. As a result, we really need a new test to
+  # check that warnings are correctly emitted.
   m <- nimbleModel(
     nimbleCode({
       y ~ dnorm(0.2 * a, sd = 2)
@@ -341,7 +353,9 @@ test_that("Laplace 1D with deterministic intermediates works", {
   cmLaplace <- cL$mLaplace
   cmLaplaceNoSplit <- cL$mLaplaceNoSplit
 
-  expect_output(opt <- cmLaplace$findMLE(), "optim does not converge for the inner optimization")
+  #expect_output(
+    opt <- cmLaplace$findMLE()
+  #, "Warning: inner optimzation had a non-zero convergence code\\. Use checkInnerConvergence\\(TRUE\\) to see details\\.")
   expect_equal(opt$par, 40, tol = 1e-4) # 40 = 4 * (1/.2) * (1/.5)
   # V[a] = 9
   # V[y] = 0.2^2 * 9 + 4 = 4.36
@@ -352,26 +366,36 @@ test_that("Laplace 1D with deterministic intermediates works", {
   # Jacobian of ahat wrt mu is 4*0.5/(4+9*0.2^2) = 0.4587156
   # Hessian of joint loglik wrt a: -(0.2^2/4 + 1/9)
   # Hessian of marginal loglik wrt param mu is -(0.2*0.5)^2/4.36 = -0.002293578
-  expect_output(summ <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE,
-                                          jointCovariance = TRUE),
-                "optim does not converge for the inner optimization")
+  cmLaplace$updateSettings(innerOptimWarning=TRUE)
+  #expect_output(
+    summ <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE,
+                              jointCovariance = TRUE)
+  #,  "optim did not converge for the inner optimization")
   expect_equal(summ$randomEffects$estimates, 20, tol = 1e-4)
   # Covariance matrix 
   vcov <- matrix(c(0, 0, 0, 1/(0.2^2/4+1/9)), nrow = 2) + matrix(c(1, 0.4587156), ncol = 1) %*% (1/0.002293578) %*% t(matrix(c(1, 0.4587156), ncol = 1))
   expect_equal(vcov, summ$vcov, tol = 1e-4)
   # Check covariance matrix for params only
-  summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
+  #expect_output(
+    summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
+   #,            "does not converge")
   expect_equal(summ2$vcov, vcov[1,1,drop=FALSE], tol=1e-4)
 
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
-  expect_output(optNoSplit <- cmLaplaceNoSplit$findMLE(), "optim does not converge for the inner optimization")
+  #expect_output(
+    optNoSplit <- cmLaplaceNoSplit$findMLE()
+  #, "Warning: inner optimzation had a non-zero convergence code\\. Use checkInnerConvergence\\(TRUE\\) to see details\\.")
   expect_equal(opt$par, optNoSplit$par, tol = 1e-2)
   expect_equal(opt$value, optNoSplit$value, tol = 1e-7)
-  check_laplace_alternative_methods(cmLaplace, cm, m, opt, expected_warning = "optim does not converge for the inner optimization")
-  check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, optNoSplit, expected_warning = "optim does not converge for the inner optimization")
+  cmLaplace$updateSettings(innerOptimWarning=TRUE) ## Turn warnings on for test.
+  check_laplace_alternative_methods(cmLaplace, cm, m, opt)#, expected_warning = "optim did not converge for the inner optimization")
+  cmLaplaceNoSplit$updateSettings(innerOptimWarning=TRUE) ## Turn warnings on for test.
+  check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, optNoSplit)#, expected_warning = "optim did not converge for the inner optimization")
 })
 
 test_that("Laplace 1D with a constrained parameter and deterministic intermediates works", {
+  ## Again (see above), the innerOptimWarning and expect_output are
+  ## defunct portions of this test.
   m <- nimbleModel(
     nimbleCode({
       y ~ dnorm(0.2 * a, sd = 2)
@@ -388,7 +412,10 @@ test_that("Laplace 1D with a constrained parameter and deterministic intermediat
   cmLaplace <- cL$mLaplace
   cmLaplaceNoSplit <- cL$mLaplaceNoSplit
   
-  expect_output(opt <- cmLaplace$findMLE(), "optim does not converge for the inner optimization")
+  #expect_output(
+    opt <- cmLaplace$findMLE()
+  #, "Warning: inner optimzation had a non-zero convergence code\\. Use checkInnerConvergence\\(TRUE\\) to see details\\.")
+  
   # V[a] = 9
   # V[y] = 0.2^2 * 9 + 4 = 4.36
   # y ~ N(0.2*0.5*mu, 4.36)
@@ -402,33 +429,56 @@ test_that("Laplace 1D with a constrained parameter and deterministic intermediat
   expect_equal(opt$hessian[1,1], -3.669725, tol = 1e-3)
   expect_equal(opt$value, dnorm(0.1*40, 0.1*40, sd = sqrt(4.36), log = TRUE))
 
-  expect_output(summ <- cmLaplace$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE,
-                                          jointCovariance = TRUE),
-                "optim does not converge for the inner optimization")
+  cmLaplace$updateSettings(innerOptimWarning=TRUE, useInnerCache=FALSE)
+  #cmLaplace$setInnerOptimWarning(TRUE)
+  #cmLaplace$setInnerCache(FALSE)
+  #expect_output(
+    summ <- cmLaplace$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE,
+                              jointCovariance = TRUE)
+   #,            "optim did not converge for the inner optimization")
   expect_equal(summ$randomEffects$estimates, 20, tol = 1e-4)
   # Covariance matrix on transformed scale
   vcov_transform <- matrix(c(0, 0, 0, 1/(0.2^2/4+1/9)), nrow = 2) + matrix(c(1, 18.34862), ncol = 1) %*% (1/3.669725) %*% t(matrix(c(1, 18.34862), ncol = 1))
   expect_equal(vcov_transform, summ$vcov, tol = 1e-3)
-  expect_output(summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE,
-                                           jointCovariance = TRUE),
-                "optim does not converge for the inner optimization")
+  #expect_output(
+    summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE,
+                               jointCovariance = TRUE)
+  #, "optim did not converge for the inner optimization")
   # Covariance matrix on original scale
   vcov <- diag(c(40,1)) %*% vcov_transform %*% diag(c(40,1))
   expect_equal(vcov, summ2$vcov, tol = 1e-3)
+
+  # Check summary based on not recomputing random effects and hessian.
+  summ.orig <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
   # Check covariance matrix for params only
-  expect_output(summ3 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE),
-                "optim does not converge for the inner optimization")
+  #expect_output(
+    summ3 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
+  #,  "optim did not converge for the inner optimization")
+
+  # Make sure that the recompute didn't change the values of the random effects or standard errors.
+  expect_equal(summ.orig$randomEffects[["estimates"]], summ3$randomEffects[["estimates"]], tol = 1e-12)
+  expect_equal(summ.orig$randomEffects[["stdErrors"]], summ3$randomEffects[["stdErrors"]], tol = 1e-12)
+
   expect_equal(summ3$vcov, vcov[1,1,drop=FALSE], tol=1e-3)
-  expect_output(summ4 <- cmLaplace$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = FALSE),
-                "optim does not converge for the inner optimization")
+  #expect_output(
+    summ4 <- cmLaplace$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
+   #,             "optim did not converge for the inner optimization")
   expect_equal(summ4$vcov, vcov_transform[1,1,drop=FALSE], tol=1e-4)
   
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
-  expect_output(optNoSplit <- cmLaplaceNoSplit$findMLE(),  "optim does not converge for the inner optimization")
+  cmLaplace$updateSettings(innerOptimWarning=TRUE)
+#  cmLaplaceNoSplit$setInnerOptimWarning(TRUE)
+  #expect_output(
+  optNoSplit <- cmLaplaceNoSplit$findMLE()
+  #,  "optim did not converge for the inner optimization")
   expect_equal(opt$par, optNoSplit$par, tol = 1e-2)
   expect_equal(opt$value, optNoSplit$value, tol = 1e-7)
-  check_laplace_alternative_methods(cmLaplace, cm, m, opt, expected_warning = "optim does not converge for the inner optimization")
-  check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, optNoSplit, expected_warning = "optim does not converge for the inner optimization")
+  cmLaplace$updateSettings(innerOptimWarning=TRUE)
+  #cmLaplace$setInnerOptimWarning(TRUE) ## Turn warnings on for test.
+  check_laplace_alternative_methods(cmLaplace, cm, m, opt)#, expected_warning = "optim did not converge for the inner optimization")
+  cmLaplaceNoSplit$updateSettings(innerOptimWarning=TRUE)
+  #cmLaplaceNoSplit$setInnerOptimWarning(TRUE) ## Turn warnings on for test.
+  check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, optNoSplit)#, expected_warning = "optim did not converge for the inner optimization")
 })
 
 test_that("Laplace 1D with deterministic intermediates and multiple data works", {
@@ -755,11 +805,16 @@ test_that("Laplace with 2x1D random effects needing joint integration works, wit
   tmbvcov[1,] <- c(20.333333, 1.1666667, 1.1666667)
   tmbvcov[2,] <- c(1.166667, 0.6651515, 0.5015152)
   tmbvcov[3,] <- c(1.166667, 0.5015152, 0.6651515)
-  expect_equal(summ$vcov, tmbvcov, tol = 1e-6)
+  expect_equal(summ$vcov, tmbvcov, tol = 1e-4)
   
   # Check covariance matrix for params only
   summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ2$vcov, tmbvcov[1,1,drop=FALSE], tol=1e-7)
+  expect_equal(summ2$vcov, tmbvcov[1,1,drop=FALSE], tol=1e-4)
+
+  #cmLaplace$setInnerCache(FALSE)
+  cmLaplace$updateSettings(useInnerCache=FALSE)
+  summ2_recomp <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
+  expect_equal(summ2$vcov, summ2_recomp$vcov, tol=1e-10)
 
   summL <- summaryLaplace(cmLaplace, opt, jointCovariance = TRUE)
   expect_equal(nrow(summL$randomEffects), 2)
@@ -868,10 +923,10 @@ test_that("Laplace with 2x1D random effects needing joint integration works, wit
   tmbvcov[2,] <- c(1.822917, 1.0380050, 0.7849117)
   tmbvcov[3,] <- c(1.822917, 0.7849117, 1.0380050)
 
-  expect_equal(summ$vcov, tmbvcov, tol = 1e-6)
+  expect_equal(summ$vcov, tmbvcov, tol = 1e-4)
   # Check covariance matrix for params only
   summ2 <- cmLaplace$summary(opt, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ2$vcov, tmbvcov[1,1,drop=FALSE], tol=1e-6)
+  expect_equal(summ2$vcov, tmbvcov[1,1,drop=FALSE], tol=1e-4)
 
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
   optNoSplit <- cmLaplaceNoSplit$findMLE() # some warnings are ok here
@@ -1123,6 +1178,8 @@ test_that("Laplace with 2x2D random effects for 2D data that need joint integrat
 })
 
 test_that("simple LME case works", {
+  # This test uses BFGS for inner and outer optimization method.
+  # nlminb results in an outer Hessian that is not negative definite.
   set.seed(1)
   g <- rep(1:10, each = 5)
   n <- length(g)
@@ -1153,19 +1210,20 @@ test_that("simple LME case works", {
   library(lme4)
   manual_fit <- lmer(y ~ x + (1 + x || g), REML = FALSE)
 
-  mLaplace <- buildLaplace(model = m)
+  mLaplace <- buildLaplace(model = m, control=list(innerOptimMethod="BFGS"))
   cm <- compileNimble(m)
   cmLaplace <- compileNimble(mLaplace, project = m)
-  opt <- cmLaplace$findMLE()
+  opt <- cmLaplace$findMLE(method="BFGS")
   nimres <- cmLaplace$summary(opt, randomEffectsStdError = TRUE)
   lme4res <- summary(manual_fit)
-  expect_equal(nimres$params$estimates[4:5], as.vector(lme4res$coefficients[,"Estimate"]), tol=1e-6)
+  expect_equal(nimres$params$estimates[4:5], as.vector(lme4res$coefficients[,"Estimate"]), tol=1e-5)
   expect_equal(nimres$params$estimates[1:3], as.data.frame(VarCorr(manual_fit))[,"sdcor"], tol = 1e-5)
-  expect_equal(nimres$params$stdErrors[4:5], as.vector(lme4res$coefficients[,"Std. Error"]), tol=1e-4)
-  expect_equal(nimres$randomEffects$estimates, as.vector(t(ranef(manual_fit)$g)), tol = 1e-5)
+  expect_equal(nimres$params$stdErrors[4:5], as.vector(lme4res$coefficients[,"Std. Error"]), tol=0.03)
+  expect_equal(nimres$randomEffects$estimates, as.vector(t(ranef(manual_fit)$g)), tol = 1e-4)
 })
 
-test_that("simple LME with correlated intercept and slope works", {
+test_that("simple LME with correlated intercept and slope works (and check with nQuad=3)", {
+  nimbleOptions(buildInterfacesForCompiledNestedNimbleFunctions=TRUE)
   set.seed(1)
   g <- rep(1:10, each = 10)
   n <- length(g)
@@ -1199,17 +1257,114 @@ test_that("simple LME with correlated intercept and slope works", {
   y <- m$y
   library(lme4)
   manual_fit <- lmer(y ~ x + (1 + x | g), REML = FALSE)
-  mLaplace <- buildLaplace(model = m)
+  mLaplace <- buildLaplace(model = m)#, control=list(innerOptimStart="last.best"))
   cm <- compileNimble(m)
   cmLaplace <- compileNimble(mLaplace, project = m)
+
+  params_in_order <- setupMargNodes(m)$paramNodes
+
+  pStart <- values(m, params_in_order)
+
+  init_llh <- cmLaplace$calcLogLik(pStart)
+  init_gr_llh <- cmLaplace$gr_logLik(pStart)
+
   opt <- cmLaplace$findMLE()
   nimres <- cmLaplace$summary(opt, randomEffectsStdError = TRUE)
+  nimsumm <- summaryLaplace(cmLaplace, opt, randomEffectsStdError = TRUE)
+
   lme4res <- summary(manual_fit)
   expect_equal(nimres$params$estimates[4:5], as.vector(lme4res$coefficients[,"Estimate"]), tol=1e-4)
   sdparams <- nimres$params$estimates[-c(4,5)]
   expect_equal(sdparams[c(1,2,4,3)], as.data.frame(VarCorr(manual_fit))[,"sdcor"], tol = 1e-3)
-  expect_equal(nimres$params$stdErrors[4:5], as.vector(lme4res$coefficients[,"Std. Error"]), tol=5e-3)
+  expect_equal(nimres$params$stdErrors[4:5], as.vector(lme4res$coefficients[,"Std. Error"]), tol=.03)
   expect_equal(nimres$randomEffects$estimates, as.vector(t(ranef(manual_fit)$g)), tol = 5e-3)
+
+  cmLaplace$updateSettings(nQuad = 3)
+  init_llh_3 <- cmLaplace$calcLogLik(pStart)
+  max_llh_3 <- cmLaplace$calcLogLik(opt$par  )
+  expect_equal(init_llh, init_llh_3, tolerance = 1e-7)
+  expect_equal(opt$value, max_llh_3, tolerance = 1e-4)
+
+  for(v in m$getVarNames()) cm[[v]] <- m[[v]]
+  cm$calculate()
+  cmLaplace$updateSettings(useInnerCache=FALSE)
+  cmLaplace$updateSettings(nQuad = 1)
+  CrunLaplaceRes <- runLaplace(cmLaplace, pStart = pStart)
+  expect_equal(opt$par, CrunLaplaceRes$MLE$par, tolerance = 1e-4)
+  expect_equal(opt$hessian, CrunLaplaceRes$MLE$hessian, tolerance = 1e-4)
+  expect_equal(nimsumm$randomEffects$estimate,
+               CrunLaplaceRes$summary$randomEffects$estimate, tolerance = 1e-4)
+  expect_equal(nimsumm$randomEffects$se,
+               CrunLaplaceRes$summary$randomEffects$se, tolerance = 1e-4)
+
+
+  for(v in m$getVarNames()) cm[[v]] <- m[[v]]
+  cm$calculate()
+  cmLaplace$updateSettings(useInnerCache=FALSE)
+  CrunLaplaceRes <- runLaplace(cmLaplace, pStart = pStart)
+  expect_equal(opt$par, CrunLaplaceRes$MLE$par, tolerance = 1e-4)
+  expect_equal(opt$hessian, CrunLaplaceRes$MLE$hessian, tolerance = 1e-4)
+  expect_equal(nimsumm$randomEffects$estimate,
+               CrunLaplaceRes$summary$randomEffects$estimate, tolerance = 1e-4)
+  expect_equal(nimsumm$randomEffects$se,
+               CrunLaplaceRes$summary$randomEffects$se, tolerance = 1e-4)
+
+})
+
+test_that("simple LME with correlated intercept and slope works through runLaplace", {
+  set.seed(1)
+  g <- rep(1:10, each = 10)
+  n <- length(g)
+  x <- runif(n)
+  m <- nimbleModel(
+    nimbleCode({
+      for(i in 1:n) {
+        y[i] ~ dnorm((fixed_int + random_int_slope[g[i], 1]) + (fixed_slope + random_int_slope[g[i], 2])*x[i], sd = sigma_res)
+      }
+      cov[1, 1] <- sigma_int^2
+      cov[2, 2] <- sigma_slope^2
+      cov[1, 2] <- rho * sigma_int * sigma_slope
+      cov[2, 1] <- rho * sigma_int * sigma_slope
+      for(i in 1:ng) {
+        random_int_slope[i, 1:2] ~ dmnorm(zeros[1:2], cov = cov[1:2, 1:2])
+      }
+      sigma_int ~ dunif(0, 10)
+      sigma_slope ~ dunif(0, 10)
+      sigma_res ~ dunif(0, 10)
+      fixed_int ~ dnorm(0, sd = 100)
+      fixed_slope ~ dnorm(0, sd = 100)
+      rho ~ dunif(-1, 1)
+    }),
+    constants = list(g = g, ng = max(g), n = n, x = x, zeros = rep(0, 2)),
+    buildDerivs = TRUE
+  )
+  params <- c("fixed_int", "fixed_slope", "sigma_int", "sigma_slope", "sigma_res", "rho")
+  values(m, params) <- c(10, 0.5, 3, 0.25, 0.2, 0.45)
+  m$simulate(m$getDependencies(params, self = FALSE))
+  m$setData('y')
+  y <- m$y
+  library(lme4)
+  manual_fit <- lmer(y ~ x + (1 + x | g), REML = FALSE)
+  mLaplace <- buildLaplace(model = m)#, control=list(innerOptimStart="last.best"))
+  cm <- compileNimble(m)
+  cmLaplace <- compileNimble(mLaplace, project = m)
+
+  pStart <- values(m, params)
+
+  res <- runLaplace(cmLaplace)
+  opt <- res$MLE
+  nimsumm <- res$summary
+
+  #opt <- cmLaplace$findMLE()
+  #nimres <- cmLaplace$summary(opt, randomEffectsStdError = TRUE)
+  #nimsumm <- summaryLaplace(cmLaplace, opt, randomEffectsStdError = TRUE)
+
+  lme4res <- summary(manual_fit)
+  expect_equal(nimsumm$params$estimate[4:5], as.vector(lme4res$coefficients[,"Estimate"]), tol=1e-4)
+  sdparams <- nimsumm$params$estimate[-c(4,5)]
+  expect_equal(sdparams[c(1,2,4,3)], as.data.frame(VarCorr(manual_fit))[,"sdcor"], tol = 1e-3)
+  expect_equal(nimsumm$params$se[4:5], as.vector(lme4res$coefficients[,"Std. Error"]), tol=.03)
+  expect_equal(nimsumm$randomEffects$estimate, as.vector(t(ranef(manual_fit)$g)), tol = 5e-3)
 })
 
 test_that("Laplace with non-empty calcNodesOther works", {
@@ -1341,14 +1496,14 @@ test_that("Laplace with 2x1D parameters (one needs transformation) and non-norma
   tmbvcov[5,] <- c(0.07943536, -0.28810783, -0.01342937, 0.05824110,  0.16979550,  0.16423022,  0.02255625)
   tmbvcov[6,] <- c(0.06797944, -1.07845809, -0.23826114, 0.03110420,  0.16423022,  0.50464751, -0.10296956)
   tmbvcov[7,] <- c(0.09118502,  0.51064309,  0.21393310, 0.08580658,  0.02255625, -0.10296956,  0.22602059)
-  expect_equal(summ$vcov, tmbvcov, tol=1e-4)
+  expect_equal(summ$vcov, tmbvcov, tol=1e-3)
   ## Stand error for sigma (original parameter)
   summ2 <- cmLaplace$summary(opt, originalScale = TRUE)
   expect_equal(summ2$params$stdErrors[2], 0.5472659, tol=1e-4)
   
   # Check covariance matrix for transformed params only
   summ3 <- cmLaplace$summary(opt, originalScale = FALSE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ3$vcov, tmbvcov[1:2,1:2], tol=1e-4)
+  expect_equal(summ3$vcov, tmbvcov[1:2,1:2], tol=1e-3)
   
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
   optNoSplit <- cmLaplaceNoSplit$findMLE()
@@ -1420,13 +1575,13 @@ test_that("Laplace with no random effects (simple linear regression) works", {
   opt <- cmLaplace$findMLE()
   summ <- cmLaplace$summary(opt)
   ## Compare results with those from TMB
-  expect_equal(opt$par, c(-0.8899436, 1.1940911, 0.5744841), tol = 1e-5)
+  expect_equal(opt$par, c(-0.8899436, 1.1940911, 0.5744841), tol = 1e-4)
   expect_equal(opt$value, -4.323288, tol = 1e-7)
   expect_equal(summ$params$stdErrors, c(0.2598061, 0.2988869, 0.1816661), tol = 1e-5)
   
   for(v in cm$getVarNames()) cm[[v]] <- m[[v]]
   optNoSplit <- cmLaplaceNoSplit$findMLE()
-  expect_equal(opt$par, optNoSplit$par, tol = 1e-2)
+  expect_equal(opt$par, optNoSplit$par, tol = 1e-4)
   expect_equal(opt$value, optNoSplit$value, tol = 1e-7)
   check_laplace_alternative_methods(cmLaplace, cm, m, opt, expected_no_re = TRUE)
   check_laplace_alternative_methods(cmLaplaceNoSplit, cm, m, expected_no_re = TRUE)
@@ -1677,20 +1832,23 @@ test_that("Laplace with crossed random effects works", {
     inits = list(beta = 20, sigma = 1, sigma_p = 1, sigma_s = 1, mus = rep(0, ns), mup = rep(0, np)),
     buildDerivs = TRUE
   )
-  mLaplace <- buildLaplace(model = m)
+  mLaplace <- buildLaplace(model = m)#, control=list(innerOptimStart = "last.best"))
   cm <- compileNimble(m)
   cmLaplace <- compileNimble(mLaplace, project = m)
+  cmLaplace$updateSettings(innerOptimMethod = "BFGS")
   opt <- cmLaplace$findMLE()
   nimres <- cmLaplace$summary(opt, randomEffectsStdError = TRUE)
   
   lme4_fit <- lmer(diameter ~ 1 + (1|plate) + (1|sample), data = Penicillin, REML = FALSE)
   lme4res <- summary(lme4_fit)
   
-  expect_equal(nimres$params$estimates[1], lme4res$coefficients[,"Estimate"], tol=1e-6)
-  expect_equal(nimres$params$estimates[c(3,4,2)], as.data.frame(VarCorr(lme4_fit))[,"sdcor"], tol = 5e-5)
-  expect_equal(nimres$params$stdErrors[1], lme4res$coefficients[,"Std. Error"], tol=5e-4)
-  expect_equal(nimres$randomEffects$estimates[25:30], as.vector(t(ranef(lme4_fit)$sample)), tol = 5e-5)
-  expect_equal(nimres$randomEffects$estimates[1:24], as.vector(t(ranef(lme4_fit)$plate)), tol = 5e-6)
+  expect_equal(nimres$params$estimates[1], lme4res$coefficients[,"Estimate"], tol=1e-3)
+  expect_equal(nimres$params$estimates[c(3,4,2)], as.data.frame(VarCorr(lme4_fit))[,"sdcor"], tol = 5e-4)
+  # Note that with innerOptimMethod "nlminb", the next check is far off, within only about 0.2
+  # on Mac, and getting a NaN on ubuntu CI tests. (Also I don't know why those differ.)
+  expect_equal(nimres$params$stdErrors[1], lme4res$coefficients[,"Std. Error"], tol=2e-3)
+  expect_equal(nimres$randomEffects$estimates[25:30], as.vector(t(ranef(lme4_fit)$sample)), tol = 1e-3)
+  expect_equal(nimres$randomEffects$estimates[1:24], as.vector(t(ranef(lme4_fit)$plate)), tol = 1e-4)
 })
 
 test_that("Laplace with nested random effects works", {
@@ -1730,7 +1888,9 @@ test_that("Laplace with nested random effects works", {
   cmLaplace <- compileNimble(mLaplace, project = m)
   ## It seems that default start values (0, 1, 1, 1) for this example do not work well 
   ## for optimisation; use c(2, 2, 2, 2) instead
-  expect_output(opt <- cmLaplace$findMLE(pStart = c(2,2,2,2)), "optim does not converge for the inner optimization")
+  #expect_output(
+    opt <- cmLaplace$findMLE(pStart = c(2,2,2,2))
+  #, "optim does not converge for the inner optimization")
   nimres <- cmLaplace$summary(opt, randomEffectsStdError = TRUE)
   
   expect_equal(nimres$params$estimates[1], lme4res$coefficients[,"Estimate"], tol = 1e-5)
@@ -1861,6 +2021,7 @@ test_that("Laplace with N(0,1) random effects works", {
   mLaplace <- buildLaplace(m, SMN)
   cm <- compileNimble(m)
   cmLaplace <- compileNimble(mLaplace, project = m)
+  cmLaplace$updateSettings(innerOptimMethod="nlminb") # findMLE will hang using BFGS
   res <- cmLaplace$findMLE(c(0,0,1))
   # TMB code in test_N01.cpp
 ##   #include <TMB.hpp>
@@ -1895,7 +2056,7 @@ test_that("Laplace with N(0,1) random effects works", {
 ## tmbrep <- sdreport(obj, getJointPrecision = TRUE)
   ## tmbvcov <- solve(tmbrep$jointPrecision)
   ##write.table(tmbvcov, file = "", sep=",",col.names = FALSE, row.names=FALSE)
-  expect_equal(res$par, c(3.1276930, 0.1645356, 1.5657498), tolerance = 1e-6 )
+  expect_equal(res$par, c(3.1276930, 0.1645356, 1.5657498), tolerance = 1e-4 )
   summ <- cmLaplace$summary(res, randomEffectsStdError=TRUE, jointCovariance=TRUE)
   ## From the write.table call just above
   ## (which is symmetric anyway, so byrow =TRUE doesn't really matter)
@@ -1909,11 +2070,63 @@ test_that("Laplace with N(0,1) random effects works", {
       c(-0.00040366417968837,-0.000258765680997534,0.00012515416885698,-0.000521588459390233,-0.000656047342397191,-9.27215078179097e-06,0.00290834901828464,0.000631332975051338),
       c(0.0188669701122331,0.031434090552776,0.28920161604805,0.154869927065379,-0.0981200179208082,0.0144354576429849,0.000631332975051331,0.283268865188007)))
 
-  expect_equal(summ$vcov, TMB_vcov[c(6:8, 1:5), c(6:8, 1:5)], tol = 1e-5)
+  expect_equal(summ$vcov, TMB_vcov[c(6:8, 1:5), c(6:8, 1:5)], tol = 1e-4)
   # Check covariance matrix for params only
   summ2 <- cmLaplace$summary(res, originalScale = TRUE, randomEffectsStdError = TRUE, jointCovariance = FALSE)
-  expect_equal(summ2$vcov, TMB_vcov[6:8,6:8], tol=1e-5)
+  expect_equal(summ2$vcov, TMB_vcov[6:8,6:8], tol=1e-4)
 })
 
+## Now that innerOptim inits has controls for method and values,
+## we need to check over these tests and functionality.
+
+## test_that("Setting Different Initial Values for Inner Optim", {
+##   m <- nimbleModel(
+##     nimbleCode({
+##       y ~ dnorm(0.2 * a, sd = 2)
+##       a ~ dnorm(0.5 * mu, sd = 3)
+##       mu ~ dnorm(0, sd = 5)
+##     }), data = list(y = 4), inits = list(a = -1, mu = 0),
+##     buildDerivs = TRUE
+##   )
+
+##   mLaplace <- buildLaplace(model = m)
+##   cm <- compileNimble(m)
+##   cL <- compileNimble(mLaplace, project = m)
+
+##   cL$setInnerOptimWarning(TRUE) ## Print Errors.
+##   cL$setInnerCache(FALSE) ## Recalculate inner optim to check starting values.
+
+##   ## Test different starting values:
+##   cL$setInnerOptimInits("zero")
+##   expect_output(cL$calcLogLik(37), "Warning: optim did not converge for the inner optimization of AGHQuad or Laplace approximation")
+
+##   cL$setInnerOptimInits("last.best")
+##   expect_output(cL$calcLogLik(37), NA)  # Small change to actually recalculate. No warning.
+
+##   set.seed(21)
+##   cL$setInnerOptimInits("random")
+##   expect_output(cL$calcLogLik(37), NA)  # Shouldn't warn.
+
+##   values(cm, "a") <- 0  ## Bad init.
+##   cL$setInnerOptimInits("model")
+##   expect_output(cL$calcLogLik(37), "Warning: optim did not converge for the inner optimization of AGHQuad or Laplace approximation")
+
+##   values(cm, "a") <- 18
+##   cL$setInnerOptimInits("model")
+##   expect_output(cL$calcLogLik(37), NA) ## Good init.
+
+##   cL$setInnerOptimInits("last")  ## Last isn't great for this new value.
+##   expect_output(cL$calcLogLik(15), "Warning: optim did not converge for the inner optimization of AGHQuad or Laplace approximation")
+
+##   ## Inspect model to see if values are updated properly after running:
+##   cL$setInnerCache(FALSE)
+##   cL$setInnerOptimInits("random")
+##   cL$calcLogLik(15)
+##   old.val <- cm$a
+##   cL$setModelValues(15)
+##   new.val <- cm$a
+##   expect_false(old.val == new.val)
+## })
+
 nimbleOptions(enableDerivs = EDopt)
 nimbleOptions(buildModelDerivs = BMDopt)
