@@ -5,8 +5,6 @@ options(warn = 1)
 nimbleVerboseSetting <- nimbleOptions('verbose')
 nimbleOptions(verbose = FALSE)
 
-context("Testing of core R functions in NIMBLE code")
-
 ## fix result_type in nimbleEigen.h
 
 cTests <- list(
@@ -423,7 +421,7 @@ seqTests <- list(
     list(name = "seq(1, 10)", expr = quote(out <- seq(1, 10)), args = list(),
          setArgVals = quote({}), outputType = quote(double(1)), checkEqual = TRUE),
     list(name = "seq(.1, 10, by = .1)", expr = quote(out <- seq(.1, 10, by = .1)), args = list(),
-         setArgVals = quote({}), outputType = quote(double(1))),
+         setArgVals = quote({}), outputType = quote(double(1)), checkEqual = TRUE), # N.B. Seemingly some change on MacOS as of 2024-06-02 created trivial non-identicalness, so check only equality.
     list(name = "seq(1, arg1, by = arg2)", expr = quote(out <- seq(1, arg1, by = arg2)), args = list(arg1 = quote(integer()), arg2 = quote(integer())),
          setArgVals = quote({arg1 <- 8; arg2 <- 2}), outputType = quote(double(1))),
     list(name = "seq(.1, 10, length.out = 11)", expr = quote(out <- seq(.1, 10, length.out = 11)), args = list(),
