@@ -522,3 +522,53 @@ test_that("Bails out for non-constant target node hyperparameters", {
     expect_error(configureRJ(conf, targetNodes = 'beta3', priorProb = 0.5))
 
 })
+
+
+test_that("configureRJ errors when a toggled sampler updates multiple nodes", {
+
+    ## with indicator variable
+    code <- nimbleCode({
+        beta0 ~ dnorm(0, sd = 100)
+        beta1 ~ dnorm(0, sd = 100)
+        sigma ~ dunif(0, 100)
+        z1 ~ dbern(psi)
+        psi ~ dbeta(1, 1)
+        for(i in 1:50) {
+            Ypred[i] <- beta0 + beta1 * z1 * x1[i]
+            Y[i] ~ dnorm(Ypred[i], sd = sigma)
+        }
+    })
+    
+    set.seed(0)
+    x1 <- runif(50, -1, 1)
+    Y <- rnorm(50, 1.5 + 2 * x1, sd = 1)
+    
+    data   <- list(Y = Y, x1 = x1)
+    inits  <- list(beta0 = 0, beta1 = 0, sigma = 1, z1 = 1, psi = 0.5)
+    
+    Rmodel <- nimbleModel(code, data=data, inits=inits)
+    conf <- configureMCMC(Rmodel)
+    conf$removeSamplers('beta1')
+    conf$addSampler(target = c('beta0', 'beta1'), type = 'RW_block')
+    expect_error(configureRJ(conf, 'beta1', indicator = 'z1'))
+
+    ## without indicator variable
+    code <- nimbleCode({
+        beta0 ~ dnorm(0, sd = 100)
+        beta1 ~ dnorm(0, sd = 100)
+        sigma ~ dunif(0, 100)
+        for(i in 1:50) {
+            Ypred[i] <- beta0 + beta1 * z1 * x1[i]
+            Y[i] ~ dnorm(Ypred[i], sd = sigma)
+        }
+    })
+
+    inits  <- list(beta0 = 0, beta1 = 0, sigma = 1)
+
+    Rmodel <- nimbleModel(code, data=data, inits=inits)
+    conf <- configureMCMC(Rmodel)
+    conf$removeSamplers('beta1')
+    conf$addSampler(target = c('beta0', 'beta1'), type = 'RW_block')
+    expect_error(configureRJ(conf, 'beta1', priorProb = 0.1))
+    
+})
